from typing import List
from pybmd.media_pool_item import MediaPoolItem


class Folder():
    """docstring for Folder."""
    def __init__(self, folder):
        self.folder = folder
        
    def get_clip_list(self) -> List[MediaPoolItem]:
        media_pool_item_list = []
        for media_pool_item in self.folder.GetClipList():
            media_pool_item_list.append(MediaPoolItem(media_pool_item))
        return media_pool_item_list

    def get_name(self) -> str:
        return self.folder.GetName()
    
    def get_sub_folder_list(self) -> List['Folder']:
        folder_list = []
        for folder in self.folder.GetSubFolderList():
            folder_list.append(Folder(folder))
        return folder_list
    
    


from typing import List
from pybmd.gallery_still_album import GalleryStillAlbum


class Gallery():
    """docstring for Gallery."""

    def __init__(self, gallery):
        self.gallery = gallery

    def get_album_name(self, gallert_still_album: GalleryStillAlbum) -> str:
        return self.gallery.GetAlbumName(gallert_still_album)

    def get_current_still_album(self) -> GalleryStillAlbum:
        return GalleryStillAlbum(self.gallery.GetCurrentStillAlbum())

    def get_gallery_still_albums(self) -> List[GalleryStillAlbum]:
        gallery_still_album_list = []
        for gallery_still_album in self.gallery.GetGalleryStillAlbums():
            gallery_still_album_list.append(
                GalleryStillAlbum(gallery_still_album))
        return gallery_still_album_list

    def set_album_name(self, gallert_still_album: GalleryStillAlbum, album_name: str) -> bool:
        return self.gallery.SetAlbumName(gallert_still_album, album_name)

    def set_current_still_album(self, gallery_still_album: GalleryStillAlbum) -> bool:
        return self.gallery.SetCurrentStillAlbum(gallery_still_album)



from enum import Enum
from os import path
from textwrap import dedent
from typing import List

from pybmd.gallerystill import GalleryStill
from pybmd.timeline_item import TimelineItem


from dataclasses import dataclass
from dataclasses import asdict


@dataclass
class ImportOptions():
    """Docstring for ImportOptions."""

    # specifies a filesystem path to search for source clips if the media is inaccessible
    # in their original path and if "ignoreFileExtensionsWhenMatching" is True
    sourceClipsPath: str = ""

    # list of Media Pool folder objects to search for source clips if the media is not present in current folder
    sourceClipsFolders: str = ""

    autoImportSourceClipsIntoMediaPool: bool = True
    ignoreFileExtensionsWhenMatching: bool = False
    linkToSourceCameraFiles: bool = False
    useSizingInfo: bool = False
    importMultiChannelAudioTracksAsLinkedGroups: bool = False
    insertAdditionalTracks: bool = True
    insertWithOffset: str = "00:00:00:00"


class TrackTpye(Enum):
    """docstring for TrackTpye."""
    AUDIO_TRACK = 'audio'
    VIDEO_TRACK = 'video'
    SUBTITLE_TRACK = 'subtitle'


def timeline_item_class_list_transfer(timeline_item_list: List[TimelineItem]) -> list:
    timeline_item_trans_list = []

    for timeline_item in timeline_item_list:
        timeline_item_trans_list.append(timeline_item.timeline_item)

    return timeline_item_trans_list


class Timeline():
    """docstring for Timeline."""

    def __init__(self, timeline):
        self.timeline = timeline

    def add_marker(self, frame_id: str, color: str, name: str, note: str, duration: str, custom_data: str) -> bool:
        return self.timeline.AddMarker(frame_id, color, name, note, duration, custom_data)

    def apply_grade_from_drx(self, path: str, grade_mode: int, timeline_items: List[TimelineItem]) -> bool:
        return self.timeline.ApplyGradeFromDRX(str(path), grade_mode, timeline_items)

    def create_compound_clip(self, timeline_items: List[TimelineItem], clipinfo: dict) -> TimelineItem:
        return TimelineItem(timeline_item=self.timeline.CreateCompoundClip(timeline_item_class_list_transfer(timeline_items), clipinfo))

    def create_fusion_clip(self, timeline_items: List[TimelineItem]) -> TimelineItem:
        return TimelineItem(timeline_item=self.timeline.CreateFusionClip(timeline_item_class_list_transfer(timeline_items)))

    def delete_marker_at_frame(self, frame_num: int) -> bool:
        return self.timeline.DeleteMarkerAtFrame(frame_num)

    def delete_marker_by_custom_data(self, custom_data: str) -> bool:
        return self.timeline.DeleteMarkerByCustomData(custom_data)

    def delete_marker_by_color(self, color: str) -> bool:
        return self.timeline.DeleteMarkerByColor(color)

    def duplicate_timeline(self, timeline_name: str):
        return Timeline(timeline=self.timeline.DuplicateTimeline(timeline_name))

    def export(self, file_name: str, export_type, export_subtype=None) -> bool:
        #file_name should be a path, not a file name.  
        # eg. file_path=os.path.join(os.path.expanduser("~"), "Desktop/Temp/sampleExp.drt")
        #     timeline.export(file_path,LOCAL_RESOLVE.EXPORT_DRT)
        return self.timeline.Export(file_name, export_type, export_subtype)

    def get_current_clip_thumbnail_image(self) -> dict:
        return self.timeline.GetCurrentClipThumbnailImage()

    def get_current_timecode(self) -> str:
        return self.timeline.GetCurrentTimecode()

    def get_current_video_item(self) -> TimelineItem:
        return TimelineItem(self.timeline.GetCurrentVideoItem())

    def get_end_frame(self) -> int:
        return self.timeline.GetEndFrame()

    def get_item_list_in_track(self, track_type: TrackTpye, index: int) -> List[TimelineItem]:
        timeline_items_list = []
        for timeline_item in self.timeline.GetItemListInTrack(
                track_type.value, index):
            timeline_items_list.append(TimelineItem(timeline_item))
        return timeline_items_list

    def get_marker_by_custom_data(self, custom_data: str) -> dict:
        return self.timeline.GetMarkerByCustomData(custom_data)

    def get_marker_custom_data(self, frame_id: int) -> str:
        return self.timeline.GetMarkerCustomData(frame_id)

    def get_markers(self) -> dict:
        return self.timeline.GetMarkers()

    def get_name(self) -> str:
        return self.timeline.GetName()

    def get_setting(self, setting_name: str) -> str:
        return self.timeline.GetSetting(setting_name)

    def get_start_frame(self) -> int:
        return self.timeline.GetStartFrame()

    def get_track_count(self, track_type: TrackTpye) -> int:
        return self.timeline.GetTrackCount(track_type.value)

    def get_track_name(self, track_type: TrackTpye, track_index: int) -> str:
        return self.timeline.GetTrackName(track_type.value, track_index)

    def grab_all_stills(self, still_frame_source: int) -> List[GalleryStill]:
        gallery_stills_list = []
        for gallery_still in self.timeline.GrabAllStills(still_frame_source):
            gallery_stills_list.append(gallery_still)
        return gallery_stills_list

    def grab_still(self) -> GalleryStill:
        return GalleryStill(self.timeline.GrabStill())

    # TODO test this
    def import_into_timeline(self, file_path: str, import_options: ImportOptions) -> bool:
        return self.timeline.ImportIntoTimeline(file_path, asdict(import_options))

    def insert_fusion_generator_into_timeline(self, generator_name: str) -> TimelineItem:
        return TimelineItem(self.timeline.InsertFusionGeneratorIntoTimeline(generator_name))

    def insert_fusion_title_into_timeline(self, title_name: str) -> TimelineItem:
        return TimelineItem(self.timeline.InsertFusionTitleIntoTimeline(title_name))

    def insert_generator_into_timeline(self, generator_name: str) -> TimelineItem:
        return TimelineItem(self.timeline.InsertGeneratorIntoTimeline(generator_name))

    def insert_oFX_generator_into_timeline(self, generator_name: str) -> TimelineItem:
        return TimelineItem(self.timeline.InsertOFXGeneratorIntoTimeline(generator_name))

    def insert_title_into_timeline(self, title_name: str) -> TimelineItem:
        return TimelineItem(self.timeline.InsertTitleIntoTimeline(title_name))

    def set_current_timecode(self, tiemcode: str) -> bool:
        return self.timeline.SetCurrentTimecode(tiemcode)

    def set_name(self, timeline_name) -> bool:
        return self.timeline.SetName(timeline_name)

    # TODO setting_name to data class
    def set_setting(self, setting_name: str, setting_value: str) -> bool:
        return self.timeline.SetSetting(setting_name, setting_value)

    def set_track_name(self, track_type: TrackTpye, track_index: int, name: str) -> bool:
        return self.timeline.SetTrackName(track_type.value, track_index, name)

    def update_marker_custom_data(self, frame_id: int, custom_data: str) -> bool:
        return self.timeline.UpdateMarkerCustomData(frame_id, custom_data)



from os import path
from pybmd.project import Project
from typing import TYPE_CHECKING, Dict, List
if TYPE_CHECKING:
    from pybmd.bmd import Bmd

DatabaseList = List[Dict]

#TODO add docstring to functions
class ProjectManager:

    project_manager = None
    _current_project = None

    def __init__(self, _local_davinci: 'Bmd.local_davinci'):
        self.project_manager = _local_davinci.GetProjectManager()
        self._current_project = self.project_manager.GetCurrentProject()


    def close_project(self, project: Project) -> bool:  
        return self.project_manager.CloseProject(project.get_self_project())

    def create_folder(self, folder_name: str) -> bool:
        return self.project_manager.CreateFolder(folder_name)

    def create_project(self, project_name: str) -> Project:
        return Project(_project=self.project_manager.CreateProject(project_name), _project_name=project_name)

    def delete_folder(self, folder_name: str) -> bool:
        return self.project_manager.DeleteFolder(folder_name)

    def delete_project(self, project_name: str) -> bool:
        return self.project_manager.DeleteProject(project_name)

    def export_project(self, project_name: str, file_path: path, with_stills_and_luts=True) -> bool:
        return self.project_manager.ExportProject(project_name, str(file_path), with_stills_and_luts)

    def get_current_database(self) -> dict:
        return self.project_manager.GetCurrentDatabase()

    def get_current_project(self) -> Project:
        return Project(_project=self._current_project, _project_name=self._current_project.GetName())

    def get_database_list(self) -> DatabaseList:
        return self.project_manager.GetDatabaseList()

    def get_folder_list_in_current_folder(self) -> List[str]:
        return self.project_manager.GetFolderListInCurrentFolder()

    def get_project_list_in_current_folder(self) -> List[str]:
        return self.project_manager.GetProjectListInCurrentFolder()

    def goto_parent_folder(self) -> bool:
        return self.project_manager.GotoParentFolder()

    def goto_root_folder(self) -> bool:
        return self.project_manager.GotoRootFolder()

    def import_project(self, file_path: path) -> bool:
        return self.project_manager.ImportPorject(str(file_path))

    def load_project(self, project_name) -> Project:
        return Project(_project=self.project_manager.LoadProject(project_name), _project_name=project_name)

    def open_folder(self, folder_name: str) -> bool:
        return self.project_manager.OpenFolder(folder_name)

    def restore_project(self, file_path: path) -> bool:
        return self.project_manager.RestoreProject(str(file_path))

    def save_project(self) -> bool:
        return self.project_manager.SaveProject()

    def set_current_database(self, database_info: dict) -> bool:
        return self.project_manager.SetCurrentDatabase(database_info)
    
# More function BELOW!

    def database_info(self, _DbType,_DbName,_IpAddress='127.0.0.1'):
        return dict(DbType=_DbType,DbName=_DbName,IpAddress=_IpAddress)
    
    def save_and_close_current_project(self)->bool:
        self.save_project()
        return self.close_project(self.get_current_project())





class MediaPoolItem():
    """docstring for MediaPoolItem."""

    def __init__(self, media_pool_item):
        self.media_pool_item = media_pool_item

    def add_flag(self, color: str) -> bool:
        return self.media_pool_item.AddFlag(color)

    def add_marker(self, frame_id: int, color: str, name: str, note: str, duration: int, custom_data: str) -> bool:
        return self.media_pool_item.AddMarker(frame_id, color, name, note, duration, custom_data)

    def clear_clip_color(self) -> bool:
        return self.media_pool_item.ClearClipColor()

    def clear_flag_color(self, color: str) -> bool:
        return self.media_pool_item.ClearFlagColor(color)

    def delete_marker_at_frame(self, frame_num: int) -> bool:
        return self.media_pool_item.DeleteMarkerAtFrame(frame_num)

    def delete_marker_by_custom_data(self, custom_data: str) -> bool:
        return self.media_pool_item.DeleteMarkerByCustomData(custom_data)

    def delete_marker_by_color(self, color: str) -> bool:
        return self.media_pool_item.DeleteMarkerByColor(color)

    def get_clip_color(self) -> str:
        return self.media_pool_item.GetClipColor()

    # property_name as data class
    def get_clip_property(self, property_name: str = "") -> str:
        return self.media_pool_item.GetClipProperty(property_name)

    def get_flag_list(self) -> list:
        return self.media_pool_item.GetFlagList()

    def get_marker_by_custom_data(self, custom_data: str) -> dict:
        return self.media_pool_item.GetMarkerByCustomData(custom_data)

    def get_marker_custom_data(self, freamid: int) -> str:
        return self.media_pool_item.GetMarkerCustomData(freamid)

    def get_markers(self) -> dict:
        return self.media_pool_item.GetMarkers()

    def get_media_id(self) -> str:
        return self.media_pool_item.GetMediaId()

    # TODO metadata_type as data class
    def get_metadata(self, metadata_type: str = "") -> str:
        return self.media_pool_item.GetMetadata(metadata_type)

    def get_name(self) -> str:
        return self.media_pool_item.GetName()

    def link_proxy_media(self, proxy_media_file_path: str) -> bool:  
        # 'proxy_media_file_path' should be absolute clip path.
        return self.media_pool_item.LinkProxyMedia(str(proxy_media_file_path))

    def replace_clip(self, file_path: str) -> bool:
        return self.media_pool_item.ReplaceClip(str(file_path))

    def set_clip_color(self, color_name: str) -> bool:
        return self.media_pool_item.SetClipColor(color_name)

    
    def set_clip_property(self, property_type: str, property_value: str) -> bool:
        return self.media_pool_item.SetClipProperty(property_type, property_value)
    
    # TODO metadata_type as data class
    def set_metadata(self, metadata_type: str, metadata_value: str) -> bool:
        return self.media_pool_item.SetMetadata(metadata_type, metadata_value)

    def unlink_proxy_media(self) -> bool:
        return self.media_pool_item.UnlinkProxyMedia()

    def updata_marker_custom_data(self, frame_id:int, custom_data:str) -> bool:
        return self.media_pool_item.UpdataMarkerCustomData(frame_id, custom_data)




import importlib.machinery
from os import path


from pybmd.media_storage import MediaStorage
from pybmd.project_manager import ProjectManager


def load_dynamic(module, module_path: str):
    loader = importlib.machinery.ExtensionFileLoader(module, module_path)
    module = loader.load_module() # type: ignore
    return module


class Bmd:

    PYLIB = "/Applications/DaVinci Resolve/DaVinci Resolve.app/Contents/Libraries/Fusion/fusionscript.so"
    APP_NAME = 'Resolve'
    IP_ADDRESS = '127.0.0.1'

    local_davinci = None

    def __init__(self,resolve_ip=IP_ADDRESS):
        self.local_davinci = self.init_davinci(davinci_ip=resolve_ip)
        
        
        #timeline exportType can be one of the following constants:
        self.EXPORT_AAF = self.local_davinci.EXPORT_AAF
        self.EXPORT_DRT=self.local_davinci.EXPORT_DRT
        self.EXPORT_EDL = self.local_davinci.EXPORT_EDL
        self.EXPORT_FCP_7_XML = self.local_davinci.EXPORT_FCP_7_XML
        self.EXPORT_FCPXML_1_3 = self.local_davinci.EXPORT_FCPXML_1_3
        self.EXPORT_FCPXML_1_4 = self.local_davinci.EXPORT_FCPXML_1_4
        self.EXPORT_FCPXML_1_5 = self.local_davinci.EXPORT_FCPXML_1_5
        self.EXPORT_FCPXML_1_6 = self.local_davinci.EXPORT_FCPXML_1_6
        self.EXPORT_FCPXML_1_7 = self.local_davinci.EXPORT_FCPXML_1_7
        self.EXPORT_FCPXML_1_8 = self.local_davinci.EXPORT_FCPXML_1_8
        self.EXPORT_HDR_10_PROFILE_A = self.local_davinci.EXPORT_HDR_10_PROFILE_A
        self.EXPORT_HDR_10_PROFILE_B = self.local_davinci.EXPORT_HDR_10_PROFILE_B
        self.EXPORT_TEXT_CSV = self.local_davinci.EXPORT_TEXT_CSV
        self.EXPORT_TEXT_TAB = self.local_davinci.EXPORT_TEXT_TAB
        self.EXPORT_DOLBY_VISION_VER_2_9 = self.local_davinci.EXPORT_DOLBY_VISION_VER_2_9
        self.EXPORT_DOLBY_VISION_VER_4_0 = self.local_davinci.EXPORT_DOLBY_VISION_VER_4_0
        
        
        #timeline exportSubtype can be one of the following enums:
        ##for exportType is EXPORT_AAF:
        self.EXPORT_AAF_NEW = self.local_davinci.EXPORT_AAF_NEW
        self.EXPORT_AAF_EXISTING = self.local_davinci.EXPORT_AAF_EXISTING
        ##for exportType is EXPORT_EDL:
        self.EXPORT_NONE = self.local_davinci.EXPORT_NONE
        self.EXPORT_CDL = self.local_davinci.EXPORT_CDL
        self.EXPORT_SDL = self.local_davinci.EXPORT_SDL
        self.EXPORT_MISSING_CLIPS = self.local_davinci.EXPORT_MISSING_CLIPS
        
    def init_davinci(self, davinci_ip):
        """init and return Davinci Resolve object

        Args:
            davinci_ip (str, optional): Defaults to LOCATION.

        """
        bmd_module = load_dynamic(
            module='fusionscript', module_path=self.PYLIB)
        return bmd_module.scriptapp(self.APP_NAME, davinci_ip)

    def get_local_davinci(self):
        return self.local_davinci

    def delete_layout_preset(self, preset_name: str) -> bool:
        """Deletes preset named preset_name.

        Args:
            preset_name (string)

        Returns:
            bool: result
        """
        return self.local_davinci.DeleteLayoutPreset(preset_name) #type: ignore

    def export_layout_preset(self, preset_name: str, preset_file_path: str) -> bool:
        """Exports preset named preset_name to path preset_file_path.

        Args:
            preset_name (str): 
            preset_file_path (str): 

        Returns:
            bool: result
        """
        return self.local_davinci.ExportLayoutPreset(preset_name, str(preset_file_path))#type: ignore

    def fusion(self):
        """Returns the Fusion object. Starting point for Fusion scripts."""
        # if I find more info about fusion I will finish this function
        return self.local_davinci.Fusion()#type: ignore

    def get_current_page(self) -> str:
        """Returns the page currently displayed in the main window

        Returns:
            str: Returned value can be one of ("media", "cut", "edit", "fusion", "color", "fairlight", "deliver", None).
        """
        return self.local_davinci.GetCurrentPage()#type: ignore

    def get_media_stroage(self) -> MediaStorage:
        """Returns the MediaStorage  object to query and act on media locations.

        Returns:
            MediaStorage: 
        """
        return MediaStorage(self.local_davinci)

    def get_product_name(self) -> str:
        """Returns product name.

        Returns:
            str: product name
        """
        return self.local_davinci.GetProductName()#type: ignore

    def get_project_manager(self) -> ProjectManager:
        """Returns the ProjectManager object for currently open database.

        Returns:
            ProjectManager: 
        """
        return ProjectManager(self.local_davinci)

    def get_version(self) -> list:
        """Returns list of product version fields in [major, minor, patch, build, suffix] format.

        Returns:
            list: list of product version fields
        """
        return self.local_davinci.GetVersion()#type: ignore

    def get_version_string(self) -> str:
        """Returns product version in "major.minor.patch[suffix].build" format.

        Returns:
            str: product version string
        """
        return self.local_davinci.GetVersionString()#type: ignore

    def import_layout_preset(self, preset_file_path: str, preset_name: str) -> bool:
        """Imports preset from path 'preset_file_path'. 
            The optional argument 'preset_name' specifies how the preset shall be named.
            If not specified, the preset is named based on the filename.
        """
        return self.local_davinci.ImportLayoutPreset(str(preset_file_path), preset_name)#type: ignore

    def load_layout_preset(self, preset_name: str) -> bool:
        """Loads UI layout from saved preset named preset_name.

        Args:
            preset_name (str): 

        Returns:
            bool
        """
        return self.local_davinci.LoadLayoutPreset(preset_name)#type: ignore

    def open_page(self, page_name: str) -> bool:
        """Switches to indicated page in DaVinci Resolve. 

        Args:
            page_name (str): Input can be one of ("media", "cut", "edit", "fusion", "color", "fairlight", "deliver").

        Returns:
            bool
        """
        return self.local_davinci.OpenPage(page_name)#type: ignore

    def quit(self):
        """Quits the Resolve App."""
        self.local_davinci.Quit()#type: ignore

    def save_layout_preset(self, preset_name: str) -> bool:
        """Saves current UI layout as a preset named preset_name.

        Args:
            preset_name (str)

        Returns:
            bool
        """
        return self.local_davinci.SaveLayoutPreset(preset_name)#type: ignore

    def update_layout_preset(self, preset_name: str) -> bool:
        """Overwrites preset named 'preset_name' with current UI layout.

        Args:
            preset_name (str)

        Returns:
            bool
        """
        return self.local_davinci.UpdateLayoutPreset(preset_name)#type: ignore



from os import path
from typing import List
from pybmd.gallery_still import GalleryStill

from enum import Enum


class StillFormats(Enum):
    """Docstring for StillFormats."""
    DPX = "dpx"
    CIN = "cin"
    TIF = "tif"
    JPG = "jpg"
    PNG = "png"
    PPM = "ppm"
    BMP = "bmp"
    XPM = "xpm"


def get_gallery_still_list_from_class_list(list: List[GalleryStill]):
    return_list = []
    for ele in list:
        return_list.append(ele.gallery_still)
    return return_list


class GalleryStillAlbum():
    """docstring for GalleryStillAlbum."""

    def __init__(self, gallery_still_album):
        self.gallery_still_album = gallery_still_album

    def delete_stills(self, gallery_stills: List[GalleryStill]) -> bool:

        gallery_still_list = get_gallery_still_list_from_class_list(
            gallery_stills)
        return self.gallery_still_album.DeleteStills(gallery_still_list)

    def export_stills(self, gallery_stills: List[GalleryStill], folder_path: str, file_prefix: str, format: StillFormats) -> bool:
        gallery_still_list = get_gallery_still_list_from_class_list(
            gallery_stills)
        return self.gallery_still_album.ExportStills(gallery_still_list, str(folder_path), file_prefix, format.value)

    def get_label(self, gallery_still: GalleryStill) -> str:
        return self.gallery_still_album.GetLabel(gallery_still)

    def get_stills(self) -> List[GalleryStill]:
        gallery_still_list = []
        for gallery_still in self.gallery_still_album.GetStills():
            gallery_still_list.append(GalleryStill(gallery_still))
        return gallery_still_list

    def set_label(self, gallery_still: GalleryStill, label: str) -> bool:
        return self.gallery_still_album.SetLabel(gallery_still, label)




from pathlib import Path
from typing import Dict, List
from pybmd.folder import Folder
from pybmd.media_pool_item import MediaPoolItem
from pybmd.timeline import Timeline
from pybmd.timeline_item import TimelineItem

from dataclasses import dataclass
from dataclasses import asdict
from multipledispatch import dispatch


@dataclass
class ClipInfo():
    """Docstring for ClipInfo."""
    media_pool_item: MediaPoolItem
    start_frame: int
    end_frame: int
    media_type: int


@dataclass
class TimelineImportOptions():
    """Docstring for TimelineImportOptions."""
    timeline_name: str
    import_source_clips: str
    source_clips_path: str
    source_flips_folders: str
    interlace_pricessing: bool


def asdict_inside_list(list: list) -> list:
    return_list = []
    for value in list:
        return_list.append(asdict(value))
    return return_list


class MediaPool():
    """docstring for MediaPool."""

    def __init__(self, media_pool):
        self.media_pool = media_pool

    def add_sub_folder(self, folder: Folder, name: str) -> Folder:
        return Folder(self.media_pool.AddSubFolder(folder, name))

    @dispatch(List[MediaPoolItem])
    def append_to_timeline(self, clips: List[MediaPoolItem]) -> List[TimelineItem]: # type: ignore
        timeline_item_list = []
        for timeline_item in self.media_pool.AppendToTimeline([clip.media_pool_item for clip in clips]):
            timeline_item_list.append(TimelineItem(timeline_item))
        return timeline_item_list

    @dispatch(List[ClipInfo])
    def append_to_timeline(self, clips: List[ClipInfo]) -> List[TimelineItem]:
        timeline_item_list = []
        for timeline_item in self.media_pool.AppendToTimeline([asdict(ClipInfo) for ClipInfo in clips]):
            timeline_item_list.append(TimelineItem(timeline_item))
        return timeline_item_list

    def create_empty_timeline(self, name) -> Timeline:
        return Timeline(self.media_pool.CreateEmptyTimeline(name))

    @dispatch(str, List[MediaPoolItem])
    def create_timeline_from_clips(self, name: str, clips: List[MediaPoolItem]) -> Timeline:  # type: ignore
        return Timeline(self.media_pool.CreateTimelineFromClips(name, [clip.media_pool_item for clip in clips]))

    @dispatch(str, List[ClipInfo])
    def create_timeline_from_clips(self, name: str, clip_infos: List[ClipInfo]) -> Timeline:
        return self.media_pool.CreateTimelineFromClips(name, asdict_inside_list(clip_infos))

    def delete_clip_mattes(self, media_pool_item: MediaPoolItem, paths: List[str]) -> bool:
        return self.media_pool.DeleteClipMattes(media_pool_item, paths)

    def delete_clips(self, clips: List[MediaPoolItem]) -> bool:
        return self.media_pool.DeleteClips([clip.media_pool_item for clip in clips])

    def delete_folders(self, subfolders: List[Folder]) -> bool:
        return self.media_pool.DeleteFolders([folder.folder for folder in subfolders])

    def delete_timelines(self, timelines: List[Timeline]) -> bool:
        return self.media_pool.DeleteTimelines([timeline.timeline for timeline in timelines])

    def export_metadata(self, file_name: str, clips: List[MediaPoolItem]) -> bool:
        return self.media_pool.ExportMetadata(file_name, [clip.media_pool_item for clip in clips])

    def get_clip_matte_list(self, media_pool_item) -> List[Path]:
        path_list = []
        for str_path in self.media_pool.GetClipMatteList(media_pool_item):
            path_list.append(Path(str_path))
        return path_list

    def get_current_folder(self) -> Folder:
        return Folder(self.media_pool.GetCurrentFolder())

    def get_root_folder(self) -> Folder:
        return Folder(self.media_pool.GetRootFolder())

    def get_timeline_matte_list(self, folder: Folder) -> List[MediaPoolItem]:
        media_pool_item_list = []
        for media_pool_item in self.media_pool.GetTimelineMatteList(folder):
            media_pool_item_list.append(MediaPoolItem(media_pool_item))
        return media_pool_item_list

    @dispatch(List[str])
    def import_media(self, file_paths: List[str]) -> List[MediaPoolItem]: # type: ignore
        media_pool_item_list = []
        for media_pool_item in self.media_pool.ImportMedia(file_paths):
            media_pool_item_list.append(MediaPoolItem(media_pool_item))
        return media_pool_item_list

    @dispatch(List[dict])
    def import_media(self, clip_info: List[dict]) -> List[MediaPoolItem]:
        media_pool_item_list = []
        for media_pool_item in self.media_pool.ImportMedia(clip_info):
            media_pool_item_list.append(MediaPoolItem(media_pool_item))
        return media_pool_item_list

    def import_timeline_from_file(self, file_path: str, import_option: TimelineImportOptions) -> Timeline:
        return Timeline(self.media_pool.ImportTimelineFromFile(str(file_path), asdict(import_option)))

    def move_clips(self, clips: List[MediaPoolItem], target_folder: Folder) -> bool:
        return self.media_pool.MoveClips([clip.media_pool_item for clip in clips], target_folder)

    def move_folders(self, folders: List[Folder], target_folder: Folder) -> bool:
        return self.media_pool.MoveFolders([folder.folder for folder in folders], target_folder)

    def relink_clips(self, media_pool_items: List[MediaPoolItem], folder_path: str) -> bool:
        return self.media_pool.RelinkClips([clip.media_pool_item for clip in media_pool_items], str(folder_path))

    def set_current_folder(self, folder: Folder) -> bool:
        return self.media_pool.SetCurrentFolder(folder)

    def unlink_clips(self, media_pool_items: List[MediaPoolItem]) -> bool:
        return self.media_pool.UnlinkClips([clip.media_pool_item for clip in media_pool_items])





from dataclasses import asdict
from dataclasses import dataclass
from typing import Any, List
from pybmd.gallery import Gallery
from pybmd.media_pool import MediaPool

from pybmd.timeline import Timeline


RenderResolution = List[dict]


@dataclass
class RenderSetting():
    """Docstring for RenderSetting."""
    SelectAllFrames: bool
    MarkIn: int
    MarkOut: int
    TargetDir: str
    CustomName: str
    UniqueFilenameStyle: int  # 0 for prefix, 1 for suffix
    ExportVideo: bool
    ExportAudio: bool
    FormatWidth: int
    FormatHeight: int
    FrameRate: float
    # (for SD resolution: "16_9" or "4_3") (other resolutions: "square" or "cinemascope")
    PixelAspectRatio: str
    VideoQuality: Any
    #  possible values for current codec (if applicable):

    #  0(int) - will set quality to automatic

    # [1 -> MAX] (int) - will set input bit rate

    # ["Least", "Low", "Medium", "High", "Best"] (String) - will set input quality level
    AudioCodec: str
    AudioBitDepth: int
    AudioSampleRate: int
    ColorSpaceTag: str  # example: "Same as Project", "AstroDesign"
    GammaTag: str  # example: "Same as Project", "ACEScct"
    ExportAlpha: bool
    # (example: "Main10"). Can only be set for H.264 and H.265.
    EncodingProfile: str
    MultiPassEncode: bool  # Can onlt be set for H.264.
    AlphaMode: int
    # 0 - Premultipled, 1 - Straight. Can only be set if "ExportAlpha" is true.
    NetworkOptimization: bool  # Only supported by QuickTime and MP4 formats.


class Project():
    """docstring for Project."""

    project = None
    project_name = None

    def __init__(self, _project, _project_name: str):
        self.project_name = _project_name
        self.project = _project

    def get_self_project(self):
        return self.project

    def add_render_job(self) -> str:
        """Adds a render job based on current render settings to the render queue. 

        Returns:
            str: A unique job id (string) for the new render job.
        """
        return self.project.AddRenderJob()

    def delete_all_render_jobs(self) -> bool:
        return self.project.DeleteAllRenderJobs()

    def delete_render_job(self, job_id: str) -> bool:
        return self.project.DeleteRenderJob(job_id)

    def get_current_render_format_and_codec(self) -> dict:
        return self.project.GetCurrentRenderFormatAndCodec()

    def get_current_render_mode(self) -> int:
        return self.project.GetCurrentRenderMode()

    def get_current_timeline(self):
        return Timeline(timeline=self.project.GetCurrentTimeline())

    def get_gallery(self,) -> Gallery:
        return Gallery(self.project.GetGallery())

    def get_media_pool(self) -> MediaPool:
        return MediaPool(self.project.GetMediaPool())

    def get_name(self) -> str:
        return self.project.GetName()

    def get_preset_list(self) -> list:
        return self.project.GetPresetList()

    def get_render_codecs(self, render_format: str) -> dict:
        return self.project.GetRenderCodecs(render_format)

    def get_render_formats(self) -> dict:
        return self.project.GetRenderFormats()

    def get_render_job_list(self) -> list:
        return self.project.GetRenderJobList()

    def get_render_job_status(self, job_id: str) -> dict:
        return self.project.GetRenderJobStatus(job_id)

    def get_render_preset_list(self):
        return self.project.GetRenderPresetList()

    def get_render_resolutions(self, format: str, codec: str) -> RenderResolution:
        #Sample: current_project.get_render_resolutions(format='mp4',codec='h264')
        return self.project.GetRenderResolutions(format, codec)

    def get_setting(self, setting_name: str) -> str:
        # call *without parameters/NoneType * to get a snapshot of all queryable properties
        return self.project.GetSetting(setting_name)

    def get_timeline_by_index(self, idx):
        return Timeline(timeline=self.project.GetTimelineByIndex(idx))

    def get_timeline_count(self) -> int:
        return self.project.GetTimelineCount()

    def is_rendering_in_progress(self) -> bool:
        return self.project.IsRenderingInProgress()

    def load_render_preset(self, preset_name) -> bool:
        return self.project.LoadRenderPreset(preset_name)

    def refresh_lut_list(self) -> bool:
        return self.project.RefreshLUTList()

    def save_as_new_render_preset(self, preset_name) -> bool:
        return self.project.SaveAsNewRenderPreset(preset_name)

    def set_current_render_format_and_codec(self, format: str, codec: str) -> bool:
        return self.project.SetCurrentRenderFormatAndCodec(format, codec)

    def set_current_render_mode(self, render_mode: int) -> bool:
        return self.project.SetCurrentRenderMode(render_mode)

    def set_current_timeline(self, timeline: Timeline) -> bool:
        return self.project.SetCurrentTimeline(timeline.timeline)

    def set_name(self, project_name) -> bool:
        return self.project.SetName(project_name)

    def set_preset(self, preset_name: str) -> bool:
        return self.project.SetPreset(preset_name)

    def set_render_setting(self, render_setting: RenderSetting) -> bool:
        return self.project.SetRenderSetting(asdict(render_setting))

    def set_setting(self, setting_name: str, setting_value: str):
        return self.project.SetSetting(setting_name, setting_value)

    def start_rendering(self, job_ids: list, is_interactive_mode=False) -> bool:
        # if job_ids==None render all queued render jobs.
        return self.project.StartRendering(job_ids, is_interactive_mode)

    def stop_rendering(self):
        return self.project.StopRendering()





from os import path
from typing import TYPE_CHECKING, List

from pybmd.media_pool_item import MediaPoolItem
if TYPE_CHECKING:
    from pybmd.bmd import Bmd


class MediaStorage:
    """docstring for MediaStorage."""

    media_storage = None

    def __init__(self, _local_davinci: 'Bmd.local_davinci'):
        """davinci media storage

        Args:
            _local_davinci (Bmd.local_davinci): davinci object
        """
        self.media_storage = _local_davinci.GetMediaStorage()

    def add_clip_mattes_to_media_pool(self, media_pool_item: MediaPoolItem, paths: List[str], stero_eye: str = None) -> bool:
        return self.media_storage.AddClipMattesToMediaPool(media_pool_item, paths, stero_eye)

    def add_item_list_to_meida_pool(self, item_path_list: List[str]) -> List[MediaPoolItem]:
        media_pool_item_list = []
        for media_pool_item in self.media_storage.AddItemListToMeidaPool(item_path_list):
            media_pool_item_list.append(MediaPoolItem(media_pool_item))
        return media_pool_item_list

    def add_timeline_mattes_to_media_pool(self, paths) -> List[MediaPoolItem]:
        media_pool_item_list = []
        for media_pool_item in self.media_storage.AddTimelineMattesToMediaPool(paths):
            media_pool_item_list.append(MediaPoolItem(media_pool_item))
        return media_pool_item_list

    def get_file_list(self, folder_list: str) -> List[str]:
        return self.media_storage.GetFileList(str(folder_list))

    def get_mounted_volume_list(self) -> List[str]:
        return self.media_storage.GetMountedVolumeList()

    def get_sub_folder_list(self, folder_path: str) -> List[str]:
        return self.media_storage.GetSubFolderList(str(folder_path))

    def reveal_in_storage(self, path: str) -> bool:
        return self.media_storage.RevealInStorage(str(path))





import string
from dataclasses import dataclass
from dataclasses import asdict
from os import path
from typing import List
from pybmd.fusion_comp import FusionComp
from pybmd.media_pool_item import MediaPoolItem

from enum import Enum

class VersionType(Enum):
    """Docstring for VersionType."""
    LOCAL = 0
    REMOTE = 1


@dataclass
class CDL_Map():
    """Docstring for CDL_Map."""
    NodeIndex: str
    Slope: str
    Offset: str
    Power: str
    Saturation: str


class TimelineItem():
    """docstring for TimelineItem."""

    def __init__(self, timeline_item):
        self.timeline_item = timeline_item

    def get_markers(self) -> dict:
        return self.timeline_item.GetMarkers()

    def add_flag(self, color: str) -> bool:
        return self.timeline_item.AddFlag(color)

    def add_fusion_comp(self) -> FusionComp:
        return FusionComp(self.timeline_item.AddFusionComp())

    def add_marker(self, frame_id: str, color: str, name: str, note: str, duration: str, custom_data: str) -> bool:
        return self.timeline_item.AddMarker(frame_id, color, name, note, duration, custom_data)

    def add_take(self, media_pool_item: MediaPoolItem, start_frame: int, end_frame: int) -> bool:
        return self.timeline_item.AddTake(media_pool_item, start_frame, end_frame)

    def add_version(self, version_name: str, version_type: VersionType) -> bool:
        return self.timeline_item.AddVersion(version_name, version_type.value)

    def clear_clip_color(self) -> bool:
        return self.timeline_item.ClearClipColor()

    def clear_flags(self, color) -> bool:
        return self.timeline_item.ClearFlags(color)

    def copy_grades(self, target_timeline_items: List['TimelineItem']) -> bool:
        timeline_item_list = []
        for timeline_item in target_timeline_items:
            timeline_item_list.append(timeline_item.timeline_item)
        return self.timeline_item.CopyGrades(timeline_item_list)

    def delete_fusion_comp_by_name(self, comp_name: str) -> bool:
        return self.timeline_item.DeleteFusionCompByName(comp_name)

    def delet_marker_at_frame(self, fram_num: int) -> bool:
        return self.timeline_item.DeletMarkerAtFrame(fram_num)

    def delete_marker_by_custom_data(self, custom_data) -> bool:
        return self.timeline_item.DeleteMarkerByCustomData(custom_data)

    def delete_marker_by_color(self, color: str) -> bool:
        return self.timeline_item.DeleteMarkerByColor(color)

    def delete_take_by_index(self, index: int) -> bool:
        return self.timeline_item.DeleteTakeByIndex(index)

    def delet_version_by_name(self, version_name: str, version_type: VersionType) -> bool:
        return self.timeline_item.DeletVersionByName(version_name, version_type.value)

    def export_fusion_comp(self, path: str, comp_index: int) -> bool:
        return self.timeline_item.ExportFusionComp(str(path), comp_index)

    def finalize_take(self) -> bool:
        return self.timeline_item.FinalizeTake()

    def get_clip_color(self) -> str:
        return self.timeline_item.GetClipColor()

    def get_current_version(self) -> dict:
        return self.timeline_item.GetCurrentVersion()

    def get_duration(self) -> int:
        return self.timeline_item.GetDuration()

    def get_end(self) -> int:
        return self.timeline_item.GetEnd()

    def get_flag_list(self) -> list:
        return self.timeline_item.GetFlagList()

    def get_fusion_comp_by_index(self, comp_index) -> FusionComp:
        return FusionComp(self.timeline_item.GetFusionCompByIndex(comp_index))

    def get_fusion_comp_by_name(self, comp_name) -> FusionComp:
        return FusionComp(self.timeline_item.GetFusionCompByName(comp_name))

    def get_fusion_comp_count(self) -> int:
        return self.timeline_item.GetFusionCompCount()

    def get_fusion_comp_name_list(self) -> list:
        return self.timeline_item.GetFusionCompNameList()

    def get_left_offset(self) -> int:
        return self.timeline_item.GetLeftOffset()

    def get_marker_by_custom_data(self, custom_data: str) -> dict:
        return self.timeline_item.GetMarkerByCustomData(custom_data)

    def get_marker_custom_data(self, frame_id: int) -> str:
        return self.timeline_item.GetMarkerCustomData(frame_id)

    def get_media_pool_item(self,) -> MediaPoolItem:
        return MediaPoolItem(self.timeline_item.GetMediaPoolItem())

    def get_name(self) -> str:
        return self.timeline_item.GetName()

    def get_property(self, property_key):
        # if no key is specified, the method returns a dictionary(python) or table(lua) for all supported keys
        return self.timeline_item.GetProperty(property_key)

    def get_right_offset(self) -> int:
        return self.timeline_item.GetRightOffset()

    def get_selected_take_index(self) -> int:
        return self.timeline_item.GetSelectedTakeIndex()

    def get_start(self) -> int:
        return self.timeline_item.GetStart()

    def get_stereo_convergence_values(self) -> dict:
        return self.timeline_item.GetStereoConvergenceValues()

    def get_stereo_left_floating_window_params(self) -> dict:
        return self.timeline_item.GetStereoLeftFloatingWindowParams()

    def get_stereo_right_floating_window_params(self) -> dict:
        return self.timeline_item.GetStereoRightFloatingWindowParams()

    def get_take_by_index(self, index) -> dict:
        return self.timeline_item.GetTakeByIndex(index)

    def get_takes_count(self) -> int:
        return self.timeline_item.GetTakesCount()

    def get_version_name_list(self, version_type: VersionType) -> list:
        return self.timeline_item.GetVersionNameList(version_type.value)

    def import_fusion_comp(self, path: str) -> FusionComp:
        return FusionComp(self.timeline_item.ImportFusionComp(str(path)))

    def load_fusion_comp_by_name(self, comp_name: str) -> FusionComp:
        return FusionComp(self.timeline_item.LoadFusionCompByName(comp_name))

    def load_version_by_name(self, version_name: str, version_type: VersionType) -> bool:
        return self.timeline_item.LoadVersionByName(version_name, version_type.value)

    def rename_fusion_comp_by_name(self, old_name: str, new_name: str) -> bool:
        return self.timeline_item.RenameFusionCompByName(old_name, new_name)

    def rename_version_by_name(self, old_name: str, new_name: str) -> bool:
        return self.timeline_item.RenameVersionByName(old_name, new_name)

    def select_take_by_index(self, index: int) -> bool:
        return self.timeline_item.SelectTakeByIndex(index)

    def set_cdl(self, cdl_map: CDL_Map) -> bool:
        return self.timeline_item.SetCdl(asdict(cdl_map))

    def set_clip_color(self, color_name) -> bool:
        return self.timeline_item.SetClipColor(color_name)

    def set_lut(self, node_index: int, lutpath: str) -> bool:
        return self.timeline_item.SetLUT(node_index, str(lutpath))
    
    ####################################################################################################################
    #add in davinci resolve 17.4.6
    def get_num_node(self) -> int:
        return self.timeline_item.GetNumNode()
    
    def get_lut(self,node_index) -> str:
        return self.timeline_item.GetLUT(node_index)
    
    
    ##########################################################################################################################



import setuptools

with open("README.md", "r") as rm:
    long_description = rm.read()

setuptools.setup(
    name="pybmd",
    version="2022.3.5rc2",
    author="wheheo",
    author_email="wheheohu@outlook.com",
    description="python libery for Davinci Resolve(Repack)",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/WheheoHu/pybmd",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)",
        "Operating System :: MacOS :: MacOS X"
    ],
    install_requires=[
        "dataclasses"
        "multipledispatch"
        ],

)
